/*
  Name: game.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 14.08.05 14:34
  Description: The main game source-file
  Game: 24h of Le Merde
*/

#include "StdAfx.h"
#include "game.h"
#include "enemy.h"
#include "stone.h"
#include "car.h"
#include "mine.h"
#include "robot.h"

void showProgress(const int seconds, const int minutes, const int hours);
inline int getDistance(int playerPosX, int playerPosY, int enemyPosX, int enemyPosY)
{
	return (playerPosX - enemyPosX) * (playerPosX - enemyPosX)
		+ (playerPosY - enemyPosY) * (playerPosY - enemyPosY);
}

bool run;

returnStruct *game()
{
	int posX = 0, posY = SCREEN_H >> 1;

	BITMAP *car = load_bitmap("gfx/car.tga", 0), *explosion = load_bitmap("gfx/explosion.tga", 0);
	if(!car)
	    error::loadBit("gfx/car.tga");
	if(!explosion)
	    error::loadBit("gfx/explosion.tga");
	sound explosionSound;
	if(!explosionSound.load("sfx/explosion.wav"))
	    error::sampleLoad("sfx/explosion.wav");

	sound motor;
	if(!motor.load("sfx/racecar.wav"))
	    error::sampleLoad("sfx/racecar.wav");
	motor.play(1, 120, 127, 1000);

	bool needsRefresh = false;
	run = true;
	int speed = 0, oldspeed = speed;
	int minutes = 30, hours = 2, seconds = 0;
	int damage = 0;
	const int maxDamage = (SCREEN_W >> 1) - 1, enemyTypes = 2, maxEnemies = 32;
	//Array of enemy pointers for the enemies
	int i;
	enemy *enemies[maxEnemies];
	for(i = 0; i < maxEnemies; ++i)
	    enemies[i] = 0;

	//Spawn a new enemy every second second (2nd second ;)
	int spawnInterval = 32 * 2;
	int spawnCountdown = 1;
	int shootCountdown = 0;
	const int maxShootCountdown = 112;
	int attackDir = 0;
	while(!global::exit && run)
	{
		while(timerCounter > 0)
		{
			if(shootCountdown)
			    --shootCountdown;
			//Get player input
			if(keyStates[KEY_START])
			{
				motor.stop();
			    showProgress(seconds, minutes, hours);
			    motor.play(1, 120 + 20 * speed, 127, 1000 + 100 * speed);
			}
			if(key[KEY_UP] && posY > car->h >> 1)
				posY -= 2;
			if(key[KEY_DOWN] && posY < SCREEN_H - (car->h >> 1) - 8)
			    posY += 2;
			if(keyStates[KEY_LEFT] && speed > -2)
			    --speed;
			if(keyStates[KEY_RIGHT] && speed < 2)
				++speed;

			posX += speed;
			if(posX < 0)
			{
				posX = 0;
				speed = 0;
			}
			if(posX > SCREEN_W - car->w)
			{
				posX = SCREEN_W - car->w;
				speed = 0;
			}
			//Adjust motor sound if necessary
			if(speed != oldspeed)
				motor.adjust(1, 120 + 20 * speed, 127, 1000 + 100 * speed);
			oldspeed = speed;

			//Move enemies
			for(i = 0; i < maxEnemies; ++i)
		    {
				if(enemies[i])
				{
					if(enemies[i]->finished())
					{
						delete enemies[i];
						enemies[i] = 0;
					}
					else
					{
					    damage += enemies[i]->move(posX, posY, car->w, car->h);
					    if(damage > maxDamage)
					        damage = maxDamage;
					}
				}
			}
			//Attack enemies
			if(keyStates[KEY_A] && !shootCountdown)
			{
				shootCountdown = maxShootCountdown;
				attackDir = 0;
				for(i = 0; i < maxEnemies; ++i)
				{
					if(enemies[i])
					    enemies[i]->attack(posX - car->w, posY, car->w, car->h);
				}
				explosionSound.play(0, 200);
			}
			if(keyStates[KEY_B] && !shootCountdown)
			{
				shootCountdown = maxShootCountdown;
				attackDir = 1;
				for(i = 0; i < maxEnemies; ++i)
				{
					if(enemies[i])
					    enemies[i]->attack(posX + car->w, posY, car->w, car->h);
				}
				explosionSound.play(0, 200);
			}

			++seconds;
			if(seconds == 32)
			{
				seconds = 0;
				++minutes;
			}
			if(minutes == 60)
			{
				minutes = 0;
				++hours;
			}
			if(hours == 24 || damage == maxDamage)
			    run = false;

			--spawnCountdown;
			if(!spawnCountdown)
			{
				spawnCountdown = spawnInterval--;
				if(spawnInterval <= 16)
				    spawnInterval = 17;
				int type = (rand() % 100) + 1;
				for(i = 0; i < maxEnemies && enemies[i]; ++i);
				if(i == maxEnemies)
					type = 0;
				//45% are stones
				if(type >= 1 && type <= 45)
					enemies[i] = new stone;
				//17% are robots
				else if(type >= 46 && type <= 62)
					enemies[i] = new robot;
				//30% are mines
				else if(type >= 63 && type <= 92)
					enemies[i] = new mine;
				//8% are other cars
				else if(type >= 93 && type <= 100)
					enemies[i] = new enemycar;
			}

			needsRefresh = true;
			--timerCounter;
		}
		if(needsRefresh)
		{
			clear_to_color(global::dbuffer, colors::white);
			for(i = 0; i < maxEnemies; ++i)
			{
				if(enemies[i])
				    enemies[i]->draw();
			}
			masked_blit(car, global::dbuffer, 0, 0, posX, posY - (car->h >> 1), car->w, car->h);
			if(shootCountdown == maxShootCountdown)
				blit(explosion, global::dbuffer, 0, 0, posX + (attackDir ? car->w : -car->w), posY - (car->h >> 1), car->w, car->h);

			rectfill(global::dbuffer, 0, SCREEN_H - 9, SCREEN_W - 1, SCREEN_H - 1, colors::black);
			rectfill(global::dbuffer, 1, SCREEN_H - 7, (SCREEN_W >> 1) - damage, SCREEN_H - 3, colors::lgray);
			textprintf_right_ex(global::dbuffer, font, SCREEN_W - 2, SCREEN_H - 8, colors::white, -1, "Time: %2i:%2i", hours, minutes);
			if(shootCountdown)
				textout_ex(global::dbuffer, font, "Reloading", (SCREEN_W >> 1) - 3, SCREEN_H - 8, colors::gray, -1);
			show();
			needsRefresh = false;
		}
	}
	//Destroy player graphics
	destroy_bitmap(car);
	destroy_bitmap(explosion);
	//Destroy enemy graphics
	if(robotgfx)
		destroy_bitmap(robotgfx);
	robotgfx = 0;
	if(robotbullet)
		destroy_bitmap(robotbullet);
	robotbullet = 0;
	if(minegfx)
		destroy_bitmap(minegfx);
	minegfx = 0;
	if(mineExplosion)
		destroy_bitmap(mineExplosion);
	mineExplosion = 0;
	if(gfx)
		destroy_bitmap(gfx);
	gfx = 0;
	if(cargfx)
		destroy_bitmap(cargfx);
	cargfx = 0;
	//Destroy enemy sfx
	mineExplosionSfx.destroy();
	robotshoot.destroy();
	//Stop player sfx
	motor.stop();

	returnStruct *retVal = new returnStruct;
	retVal->hours = hours;
	retVal->minutes = minutes;
	return retVal;
}

void showProgress(const int seconds, const int minutes, const int hours)
{
	const int MAXPROGRESS = 24 * 60 * 32;
	int progress = seconds + minutes * 32 + hours * 60 * 32;
	float percent = float(progress) / float(MAXPROGRESS);
	int position = int(float(SCREEN_W) * percent);
	int iPercent = int(100.0f * percent);
	int counter = -32;
	while(!keyStates[KEY_START] && !global::exit)
	{
		while(timerCounter > 0)
		{
			clear_to_color(global::dbuffer, colors::white);
			hline(global::dbuffer, 0, SCREEN_H >> 1, SCREEN_W - 1, colors::black);
			circlefill(global::dbuffer, position, SCREEN_H >> 1, 4, (counter < 0) ? colors::dgray : colors::lgray);
			if(counter < 0)
				textout_centre_ex(global::dbuffer, font, "-PAUSE-", SCREEN_W >> 1, 20, colors::vdgray, -1);
			textout_centre_ex(global::dbuffer, font, "Press select to quit", SCREEN_W >> 1, 30, colors::vdgray, -1);
			textprintf_centre_ex(global::dbuffer, font, SCREEN_W >> 1, 120, colors::vdgray, -1, "%i%% done...", iPercent);
			textprintf_centre_ex(global::dbuffer, font, SCREEN_W >> 1, 130, colors::vdgray, -1, "The time is: %2i:%2i", hours, minutes);
			show();
			
			++counter;
			if(counter == 31)
			    counter = -32;
			--timerCounter;
		}
		if(keyStates[KEY_SELECT])
		{
		    run = false;
		    break;
		}
	}
}
