/*
  Name: main.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 11.08.05 14:44
  Description: This file contains the core functions of the program:
		main(), init() and uninit() as well as timer routines
  Game: SH '05 template
*/

#include "StdAfx.h"
#include <ctime>
#include <fstream>
using std::time;

#include "game.h"

//Some prototypes
void init();
void uninit();
int title();
void updateHighscore(returnStruct *retVal);

//The core function of the application
int main(int argc, char **argv)
{
	init();
	while(title() && !global::exit)
	{
		returnStruct *retVal = game();
		updateHighscore(retVal);
		delete retVal;
	}
	uninit();
} END_OF_MAIN()

//timerCounterUpdater() will update timerCounter 32 times per second.
//This timer is used to make the game run at the same speed on all computers.
volatile int timerCounter = 0;
void timerCounterUpdater()
{
	timerCounter++;
} END_OF_STATIC_FUNCTION(timerCounterUpdater)

//The close button function
void closeFunc()
{
	global::exit = true;
}

//initialise allegro and some globals
void init()
{
	//initialise Allegro, timer, keyboard and sound
	allegro_init();

	if(install_timer() < 0)
		error::timer();
	if(install_keyboard() < 0)
		error::keyboard();
	if(install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) < 0)
		error::soundDriver();

	/* TODO (Any#2#): Paste Game's name */
	//Set a title for the window
	set_window_title("24 Hours Of Le Merde - Polecatsoft");

	/* TODO (Any#1#): Define color depth and resolution */
	set_color_depth(16);
	if(set_gfx_mode(GFX_AUTODETECT_WINDOWED, 320, 200, 0, 0) < 0)
	{
		error::graphicMode();
	}

	global::dbuffer = create_bitmap(SCREEN_W, SCREEN_H);
	if(!global::dbuffer)
	    error::create();
	clear(global::dbuffer);
	textout_centre_ex(global::dbuffer, font, "Loading engine, please wait...", SCREEN_W >> 1, SCREEN_H >> 1, makecol(0xFF, 0xFF, 0xFF), -1);
	show();

    //Lock Timer Function and variable
	LOCK_FUNCTION(timerCounterUpdater);
	LOCK_VARIABLE(timerCounter);

    //Set timer to 32 fps
	install_int_ex(timerCounterUpdater, BPS_TO_TIMER(32));

	//Set Charset
	set_uformat(U_ASCII);

	//Initialise rand with the time and the randomly picked number 0xABCDE
	srand(time(0) * 0xABCDE);
	
	//Set the close callback function
	set_close_button_callback(closeFunc);
	global::exit = false;

	colors::white = makecol(0xFF, 0xFF, 0xFF);
	colors::lgray = makecol(0xCE, 0xCE, 0xCE);
	colors::gray = makecol(0x9C, 0x9C, 0x9C);
	colors::dgray = makecol(0x63, 0x63, 0x63);
	colors::vdgray = makecol(0x31, 0x31, 0x31);
	colors::black = 0;
}

void uninit()
{
	destroy_bitmap(global::dbuffer);
}

//Display the doublebuffer on the screen
void show()
{
	//You can enable screenshotting here
	//if(keyStates[KEY_F9])
	//    save_bitmap("screenshot.bmp", global::dbuffer, 0);
	blit(global::dbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}

//This function was copied from my pong clone, that's why comments are in german
//Zeigt den Titelbildschirm an
int title()
{
	set_close_button_callback(closeFunc);
	//Bild laden
	BITMAP* titleBitmap = load_bitmap("gfx/title.tga", 0);
	if(!titleBitmap)
		error::loadBit("gfx/title.tga");

    returnStruct *highscore = new returnStruct;
	if(fileExists("highscore.sav"))
	{
		std::ifstream highscoreFile("highscore.sav", std::ios::binary);
		highscoreFile.read((char*)(highscore), sizeof(returnStruct));
	}
	else
	{
		highscore->hours = 0;
		highscore->minutes = 0;
	}
	textprintf_centre_ex(titleBitmap, font, 264, 175, colors::vdgray, -1, "%2i:%2i", highscore->hours, highscore->minutes);
	delete highscore;

	int returnValue = 2;

	const int KEYS = returnValue;

	while(returnValue == KEYS)
	{
		while(timerCounter > 0)
		{
			if(keyStates[KEY_START])
				returnValue = 1;
			else if(keyStates[KEY_B] || keyStates[KEY_A] || keyStates[KEY_SELECT] || global::exit)
				returnValue = 0;

			blit(titleBitmap, global::dbuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
			show();
			clear(global::dbuffer);

			--timerCounter;

			//Wenn sich der Rckgabewert gendert hat (=der Spieler hat eine Taste gedrckt),
			//Schleife beenden
			if(returnValue < KEYS)
				break;
		}
	}
	//Den Screen leeren
	clear(global::dbuffer);
	show();

	//Speicher freigeben und Wert zurckgeben
	destroy_bitmap(titleBitmap);
	return returnValue;
}

void updateHighscore(returnStruct *retVal)
{
    returnStruct *highscore = new returnStruct;
	if(fileExists("highscore.sav"))
	{
		std::ifstream highscoreFile("highscore.sav", std::ios::binary);
		highscoreFile.read((char*)(highscore), sizeof(returnStruct));
	}
	else
	{
		highscore->hours = 0;
		highscore->minutes = 0;
	}
	if(retVal->hours > highscore->hours || (retVal->hours == highscore->hours
		&& retVal->minutes > highscore->minutes))
	{
		std::ofstream highscoreFile("highscore.sav", std::ios::binary);
		highscoreFile.write((char*)(retVal), sizeof(returnStruct));
	}
}
