/*
  Name: mine.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 14.08.05 19:40
  Description: The mine is almost like a stone, but it's larger and will explode
                after you drove over it.
  Game: 24h of Le Merde
*/

#include "StdAfx.h"
#include "enemy.h"
#include "mine.h"

BITMAP *minegfx = NULL, *mineExplosion = NULL;
sound mineExplosionSfx;

mine::mine()
{
	if(!minegfx)
	    minegfx = load_bitmap("gfx/mine.tga", 0);
	if(!minegfx)
		error::loadBit("gfx/mine.tga");
	if(!mineExplosion)
	    mineExplosion = load_bitmap("gfx/mineexplosion.tga", 0);
	if(!mineExplosion)
		error::loadBit("gfx/mineexplosion.tga");
	if(!mineExplosionSfx.isActive())
		if(!mineExplosionSfx.load("sfx/mineexplosion.wav"))
		    error::sampleLoad("sfx/mineexplosion.wav");
	damaged = false;
	posX = SCREEN_W - minegfx->w;
	posY = rand() % (SCREEN_H - 9 - (minegfx->h >> 1));
}

void mine::draw()
{
	if(damaged)
		masked_blit(mineExplosion, global::dbuffer, 0, 0, posX, posY - (mineExplosion->h >> 1), mineExplosion->w, mineExplosion->h);
	else
		masked_blit(minegfx, global::dbuffer, 0, 0, posX, posY - (minegfx->h >> 1), minegfx->w, minegfx->h);
}

bool mine::finished()
{
	return (posX + minegfx->w) <= 0 || damaged;
}

int mine::move(int playerPosX, int playerPosY, int playerWidth, int playerHeight)
{
	posX -= 4;
	if(!damaged && collides(playerPosX, playerPosY, playerWidth, playerHeight >> 1, minegfx->w, minegfx->h >> 1))
	{
		mineExplosionSfx.play(0, 200);
		damaged = true;
	    return 16;
	}
	return 0;
}

//If the player uses his attack, this method is called
void mine::attack(int playerPosX, int playerPosY, int playerWidth, int playerHeight)
{
	if(!damaged && collides(playerPosX, playerPosY, playerWidth, playerHeight >> 1, minegfx->w, minegfx->h >> 1))
	{
		mineExplosionSfx.play(0, 200);
		damaged = true;
	}
}
