/*
  Name: robot.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 14.08.05 20:32
  Description: The robot will randomly move left and right and shoot
            bullets every 4 seconds
  Game: 24h of Le Merde
*/

#include "StdAfx.h"
#include "enemy.h"
#include "robot.h"

BITMAP *robotgfx = NULL, *robotbullet = NULL;
sound robotshoot;

robot::robot()
{
	if(!robotgfx)
	    robotgfx = load_bitmap("gfx/robot.tga", 0);
	if(!robotgfx)
		error::loadBit("gfx/robot.tga");
	if(!robotbullet)
	    robotbullet = load_bitmap("gfx/robotbullet.tga", 0);
	if(!robotbullet)
		error::loadBit("gfx/robotbullet.tga");
	if(!robotshoot.isActive())
	    if(!robotshoot.load("sfx/robotshoot.wav"))
			error::sampleLoad("sfx/robotshoot.wav");
	rPosX = rand() % (SCREEN_W - robotgfx->w);
	rPosY = -(robotgfx->h >> 1);
	posX = posY = -1;
	time = 32 * 6;
	mtime = 16;
	dir = (rand() & 1) ? 1 : -1;
}

void robot::draw()
{
	masked_blit(robotgfx, global::dbuffer, 0, 0, rPosX, rPosY - (robotgfx->h >> 1), robotgfx->w, robotgfx->h);
	if(posX >= 0 && posY >= 0)
	    masked_blit(robotbullet, global::dbuffer, 0, 0, posX, posY - (robotbullet->h >> 1), robotbullet->w, robotbullet->h);
}

bool robot::finished()
{
	return (rPosY + robotgfx->h) <= 0 && posY < 0 && posX < 0;
}

int robot::move(int playerPosX, int playerPosY, int playerWidth, int playerHeight)
{
	if(!time)
		--rPosY;
	else
	{
		--time;
		if(rPosY < robotgfx->w >> 1)
		    ++rPosY;
		--mtime;
		if(!mtime)
		{
			mtime = 16;
			dir = (rand() & 1) ? 1 : -1;
		}
		rPosX += dir;
		if(rPosX < 0)
		    rPosX = 0;
		if(rPosX >= SCREEN_W - robotgfx->w)
			rPosX = SCREEN_W - robotgfx->w - 1;

		if(!(time & 63))
		{
            robotshoot.play(0, 150);
            posY = robotgfx->h + (robotbullet->h >> 1);
            posX = rPosX;
		}
	}
	if(posY >= 0 || posX >= 0)
	{
		posX -= 4;
		posY += 7;
		if(posY + robotbullet->h <= 0 || posX + robotbullet->w <= 0)
			posY = posX = -1;
		else if(collides(playerPosX, playerPosY, playerWidth, playerHeight >> 1, robotbullet->w, robotbullet->h >> 1))
		{
			posX = posY = -1;
		    return 9;
		}
	}
	return 0;
}

void robot::attack(int playerPosX, int playerPosY, int playerWidth, int playerHeight)
{
	//robot is immune against attacks
}
