/*
  Name: tools.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 11.08.05 14:44
  Description: Contains useful macro functions,
  that can be used in all other files.
  Game: SH '05 template
*/

#include "StdAfx.h"
#include <string>
#include <fstream>
using namespace std;

//Returns the file size of the file filename
//Can only be used for files smaller than 4GB
unsigned int fileSize(string filename)
{
	ifstream file(filename.c_str());
	if(file.fail())
		return 0;
	file.seekg(0, ios::end);
	return file.tellg();
}

//Returns false if the file filename can not be accessed (this normally means,
//that it doesn't exists)
bool fileExists(string filename)
{
	ifstream testStream(filename.c_str());
	return !testStream.fail();
}

//Below are functions of the keyStatesClass class

//Global instance of the keyStatesClass
keyStatesClass keyStates;

//Default constructor
keyStatesClass::keyStatesClass()
{
	resetAll();
}

//The function used to access the key value id
int keyStatesClass::operator [](int id)
{
	return getKey(id);
}

//Returns the value of a key
int keyStatesClass::getKey(int id)
{
	if(id >= KEY_MAX || id < 0)
		return 0;
	if(key[id] && !pressedKeys[id])
	{
		pressedKeys[id] = true;
		return 1;
	}
	if(!key[id])
		pressedKeys[id] = false;
	return 0;
}

//Returns true if ANY key was pressed
bool keyStatesClass::anyKey()
{
	bool retVal = false;
	for(int i = 0; i < KEY_MAX; ++i)
	{
		if(getKey(i))
			retVal = true;
	}
	return retVal;
}

//Sets all array values to false
void keyStatesClass::resetAll()
{
	for(int i = 0; i < KEY_MAX; ++i)
		pressedKeys[i] = false;
}

//Assign a value to the key id
void keyStatesClass::setState(int id, int value)
{
	pressedKeys[id] = value;
}

//Below are functions of the sound class

//different states the active variable can take
enum activeStates
{
	INACTIVE = 0,
	SAMPLEACTIVE = 1,
	MIDIACTIVE = 2,
};

//Default constructor
sound::sound()
{
	sample = NULL;
	midi = NULL;
	active = INACTIVE;
	paused = false;
}

//Destructor, calls destroy()
sound::~sound()
{
	destroy();
}

//Loads the file filename and turns autopoll on for mp3s if doPoll is true
bool sound::load(string filename)
{
	destroy();
	//Load wav file
	if(
		filename[filename.length() - 3] == 'w' &&
		filename[filename.length() - 2] == 'a' &&
		filename[filename.length() - 1] == 'v')
	{
		sample = load_sample(filename.c_str());
		if(!sample)
		    return false;

		active = SAMPLEACTIVE;
	}
	//Load midi
	else if(
		(
			filename[filename.length() - 3] == 'm' &&
			filename[filename.length() - 2] == 'i' &&
			filename[filename.length() - 1] == 'd'
		)
		||
		(
			filename[filename.length() - 4] == 'm' &&
			filename[filename.length() - 3] == 'i' &&
			filename[filename.length() - 2] == 'd' &&
			filename[filename.length() - 1] == 'i'
		))
	{
		midi = load_midi(filename.c_str());
		if(!midi)
		    return false;

		active = MIDIACTIVE;
	}
	else
	    return false;
	return true;
}

//Plays the sound with the given parameters, returns false on error
void sound::play(int loop, int vol, int pan, int freq)
{
	if(active == MIDIACTIVE)
	{
		play_midi(midi, loop);
	}
	else if(active == SAMPLEACTIVE)
	{
		play_sample(sample, vol, pan, freq, loop);
	}
}

//Stops to play a sound
void sound::stop()
{
	if(active == SAMPLEACTIVE)
	{
		stop_sample(sample);
	}
	else if(active == MIDIACTIVE)
	{
		stop_midi();
	}
}

//Alters the playing method of a sound
void sound::adjust(int loop, int vol, int pan, int freq)
{
	if(active == SAMPLEACTIVE)
	{
		adjust_sample(sample, vol, pan, freq, loop);
	}
}

//Frees memory allocated and stops playing
void sound::destroy()
{
	if(sample)
	{
		destroy_sample(sample);
		sample = NULL;
	}
	if(midi)
	{
		destroy_midi(midi);
		midi = NULL;
	}
	active = INACTIVE;
}

//Is there a sound currently loaded?
int sound::isActive()
{
	return active;
}

//Shows a file selection dialogue.
//Stores a path to the selected file in path
//w and h determine the size of the dialogue
//message will be displayed on top of the dialogue
//The files will be filtered using ext, see the allegro docs for details
//Returns false if the user closed using the cancel button
bool fileSelect(const std::string message, std::string &path,
	const std::string ext, const int w, const int h)
{
	char pathBuffer[1024];
	strncpy(pathBuffer, path.c_str(), 1024);
	int retVal = file_select_ex(message.c_str(), pathBuffer, ext.c_str(), 1024,
		w, h);
	if(!retVal)
		return false;
	path = pathBuffer;
	return true;
}
