/*
  Name: tools.h
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 11.08.05 14:41
  Description: Contains useful macro functions,
  that can be used in all other files.
  Game: SH '05 template
*/

#ifndef TOOLS_H
#define TOOLS_H

#include <string>
#include <sstream>

//Returns a std::string version of n
inline std::string i2str(int n)
{
	std::ostringstream oss;
	oss << n;
	return oss.str();
}

//Returns an integer version of a std::string
//Returns -1 on error
inline int str2i(std::string s)
{
	std::istringstream oss(s);
	int r;
	if(!(oss >> r))
		r = -1;
	return r;
}

//Returns false if the file filename can not be accessed (this normally means,
//that it doesn't exists)
bool fileExists(std::string filename);

//Returns the file size of the file filename
//Can only be used for files smaller than 4GB
unsigned int fileSize(std::string filename);

//Shows a file selection dialogue.
//Stores a path to the selected file in path
//w and h determine the size of the dialogue
//message will be displayed on top of the dialogue
//The files will be filtered using ext, see the allegro docs for details
//Returns false if the user closed using the cancel button
bool fileSelect(const std::string message, std::string &path,
	const std::string ext, const int w, const int h);

//The keyStatesClass class is used to get a pressed key only once, so the user
//has to release and press the key again to get another reaction
class keyStatesClass
{
private:
	//Key array to save the keystates
	int pressedKeys[KEY_MAX];

	//Returns the value of a key
	int getKey(int i);

	//Sets all array values to false
	void resetAll();
public:
	//Default constructor
	keyStatesClass();
	//The function used to access the key value id
	int operator[](int id);
	//Assign a value to the key id
	void setState(int id, int value);
	//Returns true if ANY key was pressed
	bool anyKey();
};

//Global instance of the keyStatesClass
extern keyStatesClass keyStates;

//The sound class is a high level interface for playing .wav, .midi and .mp3
//files without any knowledge of the low level API
class sound
{
private:
	//Stores the type of data currently used (MP3, WAV or MIDI)
	int active;

	//For .wav files
	SAMPLE *sample;
	//For .midi files
	MIDI *midi;
	//Saves if doPoll was defined as true in the load() function
	bool autoPoll;
	bool paused;
public:
	//Default constructor
	sound();
	//Destructor, calls destroy()
	~sound();
	//Loads the file filename and turns autopoll on for mp3s if doPoll is true
	bool load(std::string filename);
	//Plays the sound with the given parameters, returns false on error
	void play(int loop, int vol, int pan = 127, int freq = 1000);
	//Alters the playing method of a sound
	void adjust(int loop, int vol, int pan = 127, int freq = 1000);
	//Stops to play a sound and rewinds it
	void stop();
	//Frees memory allocated and stops playing
	void destroy();
	//Is there a sound currently loaded?
	int isActive();
};

/////-- Below are macros for use with binary file handling --/////
//Returns a char pointer on i
inline char *i2ch(int &i) { return (char*)(&i); }
//Returns a char pointer on b
inline char *b2ch(bool &b) { return (char*)(&b); }
//Returns a char pointer on f
inline char *f2ch(float &f) { return (char*)(&f); }

#endif
